@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Itself ;
-$Config::Model::Itself::VERSION = '1.245';
+$Config::Model::Itself::VERSION = '2.001';
 use Mouse ;
 use Config::Model 2.064;
 use 5.010;
@@ -21,19 +21,25 @@ use File::Find ;
 use File::Path ;
 use File::Basename ;
 use Data::Compare ;
+use Path::Tiny;
+use Mouse::Util::TypeConstraints;
 
 my $logger = Log::Log4perl::get_logger("Backend::Itself");
 
+subtype 'ModelPathTiny' => as 'Object' => where { $_->isa('Path::Tiny') };
+
+coerce 'ModelPathTiny'  => from 'Str'  => via {path($_)} ;
 
 # find all .pl file in model_dir and load them...
 
 has model_object => (is =>'ro', isa =>'Config::Model::Node', required => 1) ;
-has model_dir    => (is =>'ro', isa =>'Str', required => 1 ) ;
+has cm_lib_dir    => (is =>'ro', isa => 'ModelPathTiny', required => 1, coerce => 1 ) ;
 has force_write  => (is =>'ro', isa => 'Bool', default => 0) ;
+has root_model   => (is =>'ro', isa => 'str');
 
-has modifed_classes => (
-    is =>'rw', 
-    isa =>'HashRef[Bool]', 
+has modified_classes => (
+    is =>'rw',
+    isa =>'HashRef[Bool]',
     traits => ['Hash'],
     default => sub { {} } ,
     handles => {
@@ -44,6 +50,19 @@ has modifed_classes => (
     }
 ) ;
 
+has model_dir => (
+    is => 'ro',
+    isa => 'ModelPathTiny',
+    lazy_build => 1,
+);
+
+sub _build_model_dir {
+    my $self = shift;
+    my $md = $self->cm_lib_dir->child('models');
+    $md->mkpath;
+    return $md;
+}
+
 sub BUILD {
     my $self = shift;
 
@@ -53,11 +72,11 @@ sub BUILD {
         return unless $p =~ /^class/ ;
         return if $self->class_was_changed($args{index}) ;
         $logger->info("class $args{index} was modified");
-        
+
         $self->add_modified_class($args{index}) ;
     } ;
     $self->model_object->instance -> on_change_cb($cb) ;
-    
+
 }
 
 
@@ -67,42 +86,94 @@ sub add_modified_class {
 }
 
 
+sub read_app_files {
+    my $self = shift;
+    my $force_load = shift || 0;
+    my $read_from =  shift ;
+    my $application = shift ;
+
+    my $app_dir = $read_from || $self->model_dir->parent;
+    my %apps;
+    $logger->info("reading app files from ".$app_dir);
+    foreach my $dir ( $app_dir->children(qr/\.d$/) ) {
+
+        $logger->info("reading app dir ".$dir);
+        foreach my $file ( $dir->children() ) {
+            next if $file =~ m!/README!;
+            next if $file =~ /(~|\.bak|\.orig)$/;
+            next if $application and $file->basename ne $application;
+
+            # bad categories are filtered by the model
+            my %data = ( category => $dir->basename('.d') );
+            $logger->info("reading app file ".$file);
+
+            foreach ($file->lines({ chomp => 1})) {
+                s/^\s+//;
+                s/\s+$//;
+                s/#.*//;
+                my ( $k, $v ) = split /\s*=\s*/;
+                next unless $v;
+                $data{$k} = $v;
+            }
+
+            my $appli = $file->basename;
+            $apps{$appli} = $data{model} ;
+
+            $self->model_object->load_data(
+                data => { application => { $appli => \%data } },
+                check => $force_load ? 'no' : 'yes'
+            ) ;
+        }
+    }
+
+    return \%apps;
+}
+
 sub read_all {
     my $self = shift ;
     my %args = @_ ;
 
-    my $model = delete $args{root_model} 
-      || croak __PACKAGE__," read_all: undefined root_model";
     my $force_load = delete $args{force_load} || 0 ;
+    my $read_from ;
+    my $model_dir ;
+    if ($args{read_from}) {
+        $read_from = path (delete $args{read_from});
+        die "Cannot read from unknown dir ".$read_from unless $read_from->is_dir;
+        $model_dir = $read_from->child('models');
+        die "Cannot read from unknown dir ".$model_dir unless $model_dir->is_dir;
+    }
+
+    my $apps = $self-> read_app_files($force_load, $read_from, delete $args{application});
+
+    my $root_model_arg = delete $args{root_model} || '';
+    my $model = $apps->{$root_model_arg} || $root_model_arg ;
     my $legacy = delete $args{legacy} ;
 
     croak "read_all: unexpected parameters ",join(' ', keys %args) if %args ;
 
-    my $dir = $self->model_dir ;
-    unless (-d $dir ) {
-        croak __PACKAGE__," read_all: unknown model dir $dir";
-    }
+    my $dir = $self->model_dir;
+    $dir->mkpath ;
 
     my $root_model_file = $model ;
     $root_model_file =~ s!::!/!g ;
-    
+    my $read_dir = $model_dir || $dir;
+    $logger->info("searching model files in ".$read_dir);
+
     my @files ;
-    my $wanted = sub { 
-        my $n = $File::Find::name ;
-        push @files, $n if (-f $_ and not /~$/ 
-                            and $n !~ /CVS/
-                            and $n !~ m!.(svn|orig|pod)$!
-                            and $n =~ m!$dir/$root_model_file!
+    my $wanted = sub {
+        push @files, $_ if ( $_->is_file and /\.pl$/
+                            and m!$read_dir/$root_model_file\b!
                            ) ;
     } ;
-    find ($wanted, $dir ) ;
+    $read_dir->visit($wanted, { recurse => 1} ) ;
 
     my $i = $self->model_object->instance ;
-    
+
     my %read_models ;
     my %pod_data ;
     my %class_file_map ;
 
+    my @all_models;
     for my $file (@files) {
         $logger->info("loading config file $file");
 
@@ -113,9 +184,10 @@ sub read_all {
         # @models order is important to write configuration class back in the same
         # order as the declaration
         my @models = $tmp_model -> load ( 'Tmp' , $file ) ;
+        push @all_models, @models;
 
         my $rel_file = $file ;
-        $rel_file =~ s/^$dir\/?//;
+        $rel_file =~ s/^$read_dir\/?//;
         die "wrong reg_exp" if $file eq $rel_file ;
         $class_file_map{$rel_file} = \@models ;
 
@@ -133,7 +205,7 @@ sub read_all {
                 foreach my $elt_name (keys %{$new_model->{element}}) {
                     my $moved_data = delete $new_model->{$item}{$elt_name}  ;
                     next unless defined $moved_data ;
-                    $new_model->{element}{$elt_name}{$item} = $moved_data ; 
+                    $new_model->{element}{$elt_name}{$item} = $moved_data ;
                 }
                 delete $new_model->{$item} ;
             }
@@ -146,13 +218,13 @@ sub read_all {
                 my $list  = delete $new_model -> {$what.'_list'} ;
                 my $h     = delete $new_model -> {$what} ;
                 $new_model -> {$what} = [] ;
-                map { 
-                    push @{$new_model->{$what}}, $_, $h->{$_} 
+                map {
+                    push @{$new_model->{$what}}, $_, $h->{$_}
                 } @$list ;
             }
 
             # remove hash key with undefined values
-            map { delete $new_model->{$_} unless defined $new_model->{$_} 
+            map { delete $new_model->{$_} unless defined $new_model->{$_}
                                           and $new_model->{$_} ne ''
               } keys %$new_model ;
             $read_models{$model_name} = $new_model ;
@@ -160,6 +232,8 @@ sub read_all {
 
     }
 
+    $self->{root_model} = $model || (sort @all_models)[0];
+
     # Create all classes listed in %read_models to avoid problems with
     # include statement while calling load_data
     my $model_obj = $self->model_object ;
@@ -179,7 +253,7 @@ sub read_all {
     for my $file (@files) {
         $logger->info("loading annotations from file $file");
         my $fh = IO::File->new($file) || die "Can't open $file: $!" ;
-        my @lines = $fh->getlines ;  
+        my @lines = $fh->getlines ;
         $fh->close;
         $model_obj->load_pod_annotation(join('',@lines)) ;
 
@@ -208,11 +282,11 @@ sub get_perl_data_model{
     my $class_name = $args{class_name}
       || croak __PACKAGE__," read: undefined class name";
 
-    my $class_element = $model_obj->fetch_element('class') ; 
+    my $class_element = $model_obj->fetch_element('class') ;
 
     # skip if class was deleted during edition
     return unless $class_element->defined($class_name) ;
-    
+
     my $class_elt = $class_element -> fetch_with_id($class_name) ;
 
     my $model = $class_elt->dump_as_data ;
@@ -227,6 +301,32 @@ sub get_perl_data_model{
     return $model ;
 }
 
+sub write_app_files {
+    my $self = shift;
+
+    my $app_dir = $self->cm_lib_dir;
+    my $app_obj = $self->model_object->fetch_element('application');
+
+    foreach my $app_name ( $app_obj->fetch_all_indexes ) {
+        my $app = $app_obj->fetch_with_id($app_name);
+        my $cat_dir_name = $app->fetch_element_value( name =>'category' ).'.d';
+        $app_dir->child($cat_dir_name)->mkpath();
+        my $app_file = $app_dir->child($cat_dir_name)->child($app->index_value) ;
+
+        my @lines ;
+        foreach my $name ( $app->children ) {
+            next if $name eq 'category'; # saved as directory above
+
+            my $v = $app->fetch_element_value($name); # need to spit out 0 ?
+            next unless defined $v;
+            push @lines, "$name = $v\n";
+
+        }
+        $logger->info("writing file ".$app_file);
+        $app_file->spew(@lines);
+    }
+
+}
 
 sub write_all {
     my $self = shift ;
@@ -236,15 +336,15 @@ sub write_all {
 
     croak "write_all: unexpected parameters ",join(' ', keys %args) if %args ;
 
+    $self->write_app_files;
+
     my $map = $self->{map} ;
 
-    unless (-d $dir ) {
-        mkpath($dir,0, 0755) || die "Can't mkpath $dir:$!";
-    }
+    $dir->mkpath;
 
     # get list of all classes loaded by the editor
-    my %loaded_classes 
-      = map { ($_ => 1); } 
+    my %loaded_classes
+      = map { ($_ => 1); }
         $model_obj->fetch_element('class')->fetch_all_indexes ;
 
     # remove classes that are listed in map
@@ -255,9 +355,9 @@ sub write_all {
     }
 
     # add remaining classes in map
-    my %new_map =  map { 
-        my $f = $_; 
-        $f =~ s!::!/!g; 
+    my %new_map =  map {
+        my $f = $_;
+        $f =~ s!::!/!g;
         ("$f.pl" => [ $_ ]) ;
     } keys %loaded_classes ;
 
@@ -269,21 +369,21 @@ sub write_all {
         my @data ;
         my @notes ;
         my $file_needs_write = 0;
-        
+
         # check if any a class of a file was modified
         foreach my $class_name (@{$map_to_write{$file}}) {
             $file_needs_write++ if $self->force_write or $self->class_was_changed($class_name) ;
             $logger->info("file $file class $class_name needs write ",$file_needs_write);
         }
-        
-        next unless $file_needs_write ;    
+
+        next unless $file_needs_write ;
 
         foreach my $class_name (@{$map_to_write{$file}}) {
             $logger->info("writing class $class_name");
-            my $model 
+            my $model
               = $self-> get_perl_data_model(class_name => $class_name) ;
             push @data, $model if defined $model and keys %$model;
-            
+
             my $node = $self->{model_object}->grab("class:".$class_name) ;
             push @notes, $node->dump_annotations_as_pod ;
             # remove class name from above list
@@ -292,29 +392,29 @@ sub write_all {
 
         next unless @data ; # don't write empty model
 
-        write_model_file ("$dir/$file", $self->{header}{$file}, \@notes, \@data);
+        write_model_file ($dir->child($file), $self->{header}{$file}, \@notes, \@data);
     }
-    
+
     $self->model_object->instance->clear_changes ;
 }
 
 sub write_model_snippet {
     my $self = shift ;
     my %args = @_ ;
-    my $snippet_dir = delete $args{snippet_dir} 
+    my $snippet_dir = delete $args{snippet_dir}
       || croak __PACKAGE__," write_model_snippet: undefined snippet_dir";
-    my $model_file = delete $args{model_file} 
+    my $model_file = delete $args{model_file}
       || croak __PACKAGE__," write_model_snippet: undefined model_file";
     croak "write_model_snippet: unexpected parameters ",join(' ', keys %args) if %args ;
 
     my $model = $self->model_object->dump_as_data ;
     # print (Dumper( $model)) ;
 
-    my @raw_data = @{$model->{class}} ;
+    my @raw_data = @{$model->{class} || []} ;
     while (@raw_data) {
         my ( $class , $data ) = splice @raw_data,0,2 ;
         $data ->{name} = $class ;
- 
+
         # does not distinguish between notes from underlying model or snipper notes ...
         my @notes = $self->model_object->grab("class:$class")->dump_annotations_as_pod ;
         my $class_dir = $class.'.d';
@@ -328,17 +428,17 @@ sub write_model_snippet {
 sub read_model_snippet {
     my $self = shift ;
     my %args = @_ ;
-    my $snippet_dir = delete $args{snippet_dir} 
+    my $snippet_dir = delete $args{snippet_dir}
       || croak __PACKAGE__," write_model_snippet: undefined snippet_dir";
-    my $model_file = delete $args{model_file} 
+    my $model_file = delete $args{model_file}
       || croak __PACKAGE__," read_model_snippet: undefined model_file";
 
     croak "read_model_snippet: unexpected parameters ",join(' ', keys %args) if %args ;
 
     my @files ;
-    my $wanted = sub { 
+    my $wanted = sub {
         my $n = $File::Find::name ;
-        push @files, $n if (-f $_ and not /~$/ 
+        push @files, $n if (-f $_ and not /~$/
                             and $n !~ /CVS/
                             and $n !~ m!.(svn|orig|pod)$!
                             and $n =~ m!\.d/$model_file!
@@ -350,7 +450,7 @@ sub read_model_snippet {
 
     foreach my $load_file (@files) {
         $logger->info("trying to read snippet $load_file");
-    
+
         my $snippet = do $load_file ;
 
         unless ($snippet) {
@@ -369,7 +469,7 @@ sub read_model_snippet {
         # load annotations
         $logger->info("loading annotations from snippet file $load_file");
         my $fh = IO::File->new($load_file) || die "Can't open $load_file: $!" ;
-        my @lines = $fh->getlines ;  
+        my @lines = $fh->getlines ;
         $fh->close;
         $self->model_object->load_pod_annotation(join('',@lines)) ;
     }
@@ -489,7 +589,7 @@ sub get_dot_diagram {
             $elt_list .= "- $elt_name ($type$of)\\n";
         }
 
-        $dot .= $d_class 
+        $dot .= $d_class
              .  qq! [shape=box label="$class_name\\n$elt_list"];\n!
              .  $use . "\n";
 
@@ -560,7 +660,7 @@ Config::Model::Itself - Model editor for Config::Model
 
 =head1 VERSION
 
-version 1.245
+version 2.001
 
 =head1 SYNOPSIS
 
@@ -578,20 +678,19 @@ version 1.245
  # structure containing the model to be edited
  my $rw_obj = Config::Model::Itself -> new(
     model_object => $meta_root,
-    model_dir => '/path/to/model_files' ,
+    model_dir => '/path/to/model_files', # can be a Path::Tiny object
  ) ;
 
  # now load the model to be edited
  $rw_obj -> read_all( ) ;
 
  # For Curses UI prepare a call-back to write model
- my $wr_back = sub { $rw_obj->write_all();
+ my $wr_back = sub { $rw_obj->write_all(); }
 
  # create Curses user interface
- my $dialog = Config::Model::CursesUI-> new
-      (
-       store => $wr_back,
-      ) ;
+ my $dialog = Config::Model::CursesUI-> new (
+      store => $wr_back,
+ ) ;
 
  # start Curses dialog to edit the mode
  $dialog->start( $meta_model )  ;
@@ -630,14 +729,15 @@ dedicated to read and write a set of model files.
 
 Creates a new read/write handler. This handler is dedicated to the
 C<model_object> passed with the constructor. This parameter must be a
-L<Config::Model::Node> class.
+L<Config::Model::Node> class. C<model_dir> is either a C<Path::Tiny> object
+or a string.
 
 =head2 Methods
 
 =head1 read_all (  root_model => ... , [ force_load => 1 ] )
 
 Load all the model files contained in C<model_dir> and all its
-subdirectories. C<root_model> is used to filter the classes read. 
+subdirectories. C<root_model> is used to filter the classes read.
 
 Use C<force_load> if you are trying to load a model containing errors.
 
@@ -684,7 +784,7 @@ dashed lines. The name of the element is attached to the dashed line.
 
 =head1 SEE ALSO
 
-L<Config::Model>, L<Config::Model::Node>,
+L<Config::Model>, L<Config::Model::Node>, L<Path::Tiny>
 
 =head1 AUTHOR
 
