@@ -26,7 +26,7 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Itself::TkEditUI ;
-$Config::Model::Itself::TkEditUI::VERSION = '1.245';
+$Config::Model::Itself::TkEditUI::VERSION = '2.001';
 use strict;
 use warnings ;
 use Carp ;
@@ -47,22 +47,23 @@ sub ClassInit {
 sub Populate { 
     my ($cw, $args) = @_;
 
-    my $model_dir    = delete $args->{-model_dir} ;
-    my $model_name   = delete $args->{-model_name} ;
+    my $cm_lib_dir    = (delete $args->{-cm_lib_dir})."/models" ;
+    my $model_name   = delete $args->{-model_name} || '';
     my $root_dir     = delete $args->{-root_dir} ; # used to test the edited model
 
-    $args->{'-title'} ||= "config-model-edit $model_name" ;
+    $args->{'-title'} ||= "cme meta edit $model_name" ;
 
     $cw->SUPER::Populate($args) ;
 
-    my $items = [[ qw/command test -command/, sub{ $cw->test_model }],
-		] ;
+    my $items = [[ qw/command test -command/, sub{ $cw->test_model }] ] ;
 
     my $model_menu = $cw->{my_menu}->cascade(-label => 'Model',
 					     -menuitems => $items) ;
-    $cw->{model_dir} = $model_dir ;
+    $cw->{cm_lib_dir} = $cm_lib_dir ;
     $cw->{model_name} = $model_name ;
     $cw->{root_dir} = $root_dir ;
+
+    $cw->show_message("Add a name in Class to create your model") unless $model_name;
 }
 
 sub test_model {
@@ -97,7 +98,7 @@ sub _launch_test {
     $testw->destroy if defined $testw and Tk::Exists($testw);
 
     # need to read test model from where it was written...
-    my $model = Config::Model -> new(model_dir => $cw->{model_dir}) ;
+    my $model = Config::Model -> new(model_dir => $cw->{cm_lib_dir}) ;
 
     # keep a reference on this object, otherwise it will vanish at the end of this block.
     $cw->{test_model} =  $model ;
